﻿using Upreader.Usenet.Nntp.Encodings;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Text;
using System.Collections.Generic;

namespace Upreader.Usenet.Nntp.Tests
{
    [TestClass]
    public class MetaLineStreamTest
    {

        public class ConcreteMetaLineStream : MetaLineStream
        {
            public ConcreteMetaLineStream(Stream dataStream, string prefix, Encoding encoding)
                : base(dataStream, prefix, encoding)
            {
                MetaLines = new List<string>();
            }

            protected override void OnMetaLineFound(string line)
            {
                MetaLines.Add(line);

                if (MetaLineFound != null)
                    MetaLineFound(this, EventArgs.Empty);
            }

            public List<string> MetaLines { get; private set; }

            public event EventHandler MetaLineFound;
        }

        protected virtual ConcreteMetaLineStream CreateMetaLineStream(string line)
        {
            Stream dataStream = new MemoryStream(Encoding.ASCII.GetBytes(line));

            ConcreteMetaLineStream target = new ConcreteMetaLineStream(dataStream, "=test", Encoding.ASCII);
            return target;
        }

        [TestMethod]
        public void Read_SingleMetaLine_CanBeFound()
        {
            ConcreteMetaLineStream target = CreateMetaLineStream("hello\r\n=test world\r\n"); 
            byte[] buffer = new byte[100]; // random buffer 
            int offset = 0; 
            int count = buffer.Length;

            bool metaLineFound = false;

            target.MetaLineFound += delegate
            {
                metaLineFound = true;
            };

            target.Read(buffer, offset, count);

            Assert.IsTrue(metaLineFound);
        }


        [TestMethod]
        public void Read_SimpleContent_CountBytesSucceeds()
        {
            ConcreteMetaLineStream target = CreateMetaLineStream("hello\r\n=test world\r\n");
            byte[] buffer = new byte[100]; // random buffer 
            int offset = 0;
            int count = buffer.Length;
            int expected = 18;           

            int actual = target.Read(buffer, offset, count);

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void Read_DoubleMetaLine_CanBeFound()
        {
            ConcreteMetaLineStream target = CreateMetaLineStream("hello\r\n=test wordld\r\n=test world\r\n");
            byte[] buffer = new byte[100]; // random buffer 
            int offset = 0;
            int count = buffer.Length;

            int metaLineFound = 0;

            target.MetaLineFound += delegate
            {
                metaLineFound++;
            };

            target.Read(buffer, offset, count);

            Assert.AreEqual(metaLineFound, 2);
        }

        [TestMethod]
        public void Read_SimpleMetaLine_ReadsAllBytes()
        {
            ConcreteMetaLineStream target = CreateMetaLineStream("hello\r\n=test world\r\n");
            byte[] buffer = new byte[100]; // random buffer 
            int offset = 0;
            int count = buffer.Length;

            int total = target.Read(buffer, offset, count);

            Assert.AreEqual(total, 18);
        }

        [TestMethod]
        public void Read_StartWithLineEnd_ReadsAllMetaLines()
        {
            ConcreteMetaLineStream target = CreateMetaLineStream("\r\n=test part\r\n=test begin\r\nsn]-***");
            byte[] buffer = new byte[200]; // random buffer 
            int offset = 0;
            int count = buffer.Length;

            int total = target.Read(buffer, offset, count);

            Assert.AreEqual(target.MetaLines.Count, 2);

        }
    }
}
