﻿using Upreader.Usenet.Nntp.Encodings;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Text;

namespace Upreader.Common.Tests
{
    [TestClass]
    public class RecordStreamTest
    {
        private static Stream CreateStreamFromString(string text)
        {
            Stream stream = new MemoryStream(Encoding.UTF8.GetBytes(text));
            return stream;
        }

        [TestMethod]
        public void RecordPlayback_Fully_Succeeds()
        {
            string originalText = "Hello world";

            Stream stream = CreateStreamFromString(originalText);
            RecordStream target = new RecordStream(stream); 
            target.Record();

            StreamReader reader1 = new StreamReader(target);
            string test1 = reader1.ReadLine();
            target.Playback();
            StreamReader reader2 = new StreamReader(target);
            string test2 = reader2.ReadLine();

            Assert.AreEqual(test1, test2);
            Assert.AreEqual(test1, originalText);
        }

        [TestMethod]
        public void RecordPlayback_Partually_Succeeds()
        {
            string originalText = "Hello" + Environment.NewLine + "world";

            Stream stream = CreateStreamFromString(originalText);
            RecordStream target = new RecordStream(stream); 
            target.Record();

            StreamReader reader1 = new StreamReader(target);
            string test1 = reader1.ReadLine();

            target.Playback();
            StreamReader reader2 = new StreamReader(target);
            string test2 = reader2.ReadLine();

            target.Playback();
            StreamReader reader3 = new StreamReader(target);
            string test3 = reader3.ReadLine();
            test3 = reader3.ReadLine();

            Assert.AreEqual(test1, test2);
            Assert.AreEqual(test2, "Hello");
            Assert.IsTrue(test3.StartsWith("world"));
        }
    }
}
